'
' Toshiba T6963 Graphic LCD command tests
'
' For 16-bit core devices using Proton Version 3.08 onwards
' To use this with a 14-bit core device remove all references to String variables
'
' The program will cycle through the standard graphic LCD commands
' View the test progress via the serial terminal set to 9600 baud
'
    Device = 18F452
    Declare Xtal = 20
    Include "BUFFER_SERIE.inc"
    Declare Float_Display_Type = LARGE                    ' Use more accurate floating point display
    Declare LCD_Type = Toshiba                            ' Target a Toshiba T6963 graphic LCD
'
' LCD pin assignments
'
    Declare LCD_DTPort = PORTD                            ' Data port of the LCD
    Declare LCD_WRPin = PORTE.2                           ' WR line of the LCD
    Declare LCD_RDPin = PORTE.1                           ' RD line of the LCD
    Declare LCD_CEPin = PORTE.0                           ' CE line of the LCD
    Declare LCD_CDPin = PORTA.1                           ' CD line of the LCD
    Declare LCD_RSTPin = PORTA.0                          ' Reset line of the LCD (Optional)
'
' LCD characteristics
'
    Declare LCD_Text_Pages = 2                            ' Choose two text pages
    Declare LCD_RAM_Size = 8192                           ' Amount of RAM within display
    Declare LCD_X_Res = 240                               ' Horizontal resolution
    Declare LCD_Y_Res = 128                                ' Vertical resolution
    Declare LCD_Font_Width = 8                            ' Choose a 6 pixel font
    Declare LCD_Text_Home_Address = 0                     ' Ensure that text RAM starts at address 0
'
' Setup the USART for the demo
'
    Declare Hserial_Baud = 38400                         ' Set baud rate to 9600
    Declare Hserial_RCSTA = %10010000                     ' Enable serial port and continuous receive
    Declare Hserial_TXSTA = %00100100                     ' Enable transmit and asynchronous mode
    Declare Hserial_Clear = On                            ' Enable Error clearing on received characters
'
    Declare All_Digital = True                  ' Set PORTA and PORTE to digital mode
    

Declare Keypad_Port PORTB
    
    
'-----------------------------------------------------------
Symbol CR  =               $0D
Symbol LF  =               $0A
Symbol CTRL_Q =            $11
Symbol CTRL_C =            $03
Symbol BKSPACE_CHAR =      $08
Symbol ABORT_CHAR   =     CTRL_C
Symbol QUIT_APPEND  =     CTRL_Q
Symbol QUANTA = 5.0 / 1024		' Calculate the quantising value based on 10-bits with 5 Volts
Symbol STRLENG  =  39             'longeur de la chaine de commande 
'-----------------------------------------------------------



' Create some variables for the demo
'
    Dim I       As Byte                ' varible de boucle passe partout
    Dim U       As Byte                ' varible de boucle passe partout
    Dim PRSI    As Byte                ' poiteur dans la reception serie
    Dim ECHO    As Bit                 ' definie echo sur transmition serie O/N
    Dim hold As Byte
    
    Dim Var1 As Byte
    Dim Flt1 As Float
    Dim Dwd1 As Dword
    Dim Serial  As Byte                ' Variable de reception d un octet en serie
    Dim String1 As String * 20
    Dim Cmd_Str     As String * (STRLENG+1)     ' Chaine de commande serie
    Dim DestString  As String * (STRLENG+1)     ' travail sur la chaine de commande serie
'-------------------------------------------------------------
' The main program starts here
   Clear
    DelayMS 200                                 ' Wait for things to stabilise
    INIT_USART_INTERRUPT
    CLEAR_SERIAL_BUFFER                    ' effacement de toutes les donnees presente dans le buffer.
     HSerOut ["Test 1\r"]
    
        Cls                                     ' Clear text and graphics area of teh LCD
        Flt1 = 3.1456
        Dwd1 = 123456789
        String1 = "AxeNUM PRO"
        Print " Graphic Interface"
        Print At 1,0,String1
        Print At 3,0,"Float ",Dec Flt1
        Print At 4,0,"Dword ",Dec Dwd1
        Print At 5,0,Display_Text

        
        
        

Dim KEY As Byte

MAIN1: 



GoSub _HrsInBuffLen_Sub
If _LENGTH < 500  Then      ' ici le buffe ce remet en marche (vieux code a 500)
  If  _BUFF = 1 Then 
   HRSOut 17 '                             ,17           XOnn
   _BUFF=0
  EndIf
EndIf  

                               
If STOPREQ =1 Then                       ' si un arret a ete demander 
    STOPREQ= 0                                        ' mise a 0 du drapeau d arret
     HRSOut "Stop" 
    CLEAR_SERIAL_BUFFER                    ' effacement de toutes les donnees presente dans le buffer.
    HRSOut 17                              ',17   Xonn
    _BUFF=0
    GoSub STARTTIMER                       ' mise a 0 des secondes et remise en marche du timer
EndIf

    
HSerIn 0, Sernot , [Serial]

    If ECHO=1 Then                         ' si mode echo on revoi le caractere 
    HRSOut Serial           
    EndIf

Cmd_Str [PRSI] = Serial                    'Mise en memoire du carractere dans la chaine 
Inc PRSI                                   'Incrementation du pointeur 
    If PRSI>=STRLENG Then                  'si plus grand que la chaine
        PRSI=0                                 'mise a 0 du pointeur
        GoSub NULSTRING                        'mise a 0 de la chaine
        HRSOut "Error len string",13                          'envoi d'une erreure
        GoTo MAIN1                             'retour rapide a la reception de donnees
    EndIf

                                        'selection des diferante commande assci
Select Serial 
      
    Case  BKSPACE_CHAR                      'si reception d un back-space
        Cmd_Str[PRSI]= 0                        'supression du back-space
        Dec PRSI                                '-1 sur le pointeur
        If  PRSI>0 Then                         'si pas a 0
        Cmd_Str[PRSI]= 0                        'supresion du caractaire precedent
        Dec PRSI                                '-1 sur le pointeur
        EndIf
        GoTo Sernot                             'retour rapide a la reception de donnees
    Case ABORT_CHAR                         'si reception annulation
        PRSI=0                                  'mise a 0 du pointeur
        GoSub NULSTRING                         'mise a 0 de la chaine
        GoTo Sernot                             'retour rapide a la reception de donnees
    Case CR
        PRSI=0
        GoTo Traitement

EndSelect

Sernot:
GoTo MAIN1    
    
Traitement: 
Cls
Print At 8,1, Cmd_Str
HRSOut "String : ", Cmd_Str
GoTo MAIN1  

STOPTIMER:
T1CON.0 = 0                     'stop the timer
Return

STARTTIMER:
T1CON.0 = 1                     'start the timer   1= star 
SECONDE = 0
Return

NULSTRING:
For U=0 To STRLENG
Cmd_Str[U]=0
Next U
Return



Display_Text:
    CData "Cdata Test ",0


